/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.templates.freemarkerimpl;

import freemarker.template.*;
import com.revusky.oreo.*;
import com.revusky.niggle.servlet.ServletInteraction;
import com.revusky.oreo.util.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * A freemarker template model specifically designed
 * to "wrap" a niggle data record.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */
public class RecordWrapper extends LazyHash {
    protected Record rec;

    public void setRecord(Record rec) {
        this.rec = rec;
    }

    public boolean isEmpty() throws TemplateModelException
    {
        return rec == null && super.isEmpty();
    }

    public TemplateModel get(String key) throws TemplateModelException {
        if (CASE_INSENSITIVE) {
            key = key.toLowerCase();
        }
        TemplateModel result = super.get(key);
        if (result != null)
            return result;
        if (rec!=null) {
            result = reflectionCheck(key);
            if (result == null) {
                if (key.equals(DefaultRecord.TYPE_KEY)) {
                    result = createWrapper(rec.getType());
                }
                else {
                    FieldDescriptor field = rec.getMetadata().getField(key);
                    if (field != null)
                        result = getWrappedFieldValue(field);
                }
            }
        }
        put(key, result);
        return result;
    }


    /**
     * wrap the field value as a Freemarker TemplateModel
     * You can override this method in a subclass if 
     * if you want special (ad-hoc) treatment of a given
     * field.
     */
    protected TemplateModel getWrappedFieldValue(FieldDescriptor field) {
        Object fieldValue = rec.get(field);
        return createWrapper(fieldValue);
    }

    /**
     * This method allows us to automatically expose "variables" via 
     * the getXXX naming patterns within record subtype classes.
     */
    protected TemplateModel reflectionCheck(String key) {
        TemplateModel result = null;
        if (!Util.hasContent(key))
            return null;
        String methodName = "get" +
                            Character.toUpperCase(key.charAt(0))
                            + key.substring(1);
        String methodLookupName = rec.getClass().getName() + "|" + methodName;
        Method m = null;
        try {
            if (methodLookup.containsKey(methodLookupName))
                m = (Method) methodLookup.get(methodLookupName);
            else
                m = rec.getClass().getMethod(methodName, EMPTY_CLASS_ARRAY);
            if (m != null)
                result = createWrapper(m.invoke(rec, EMPTY_CLASS_ARRAY));
        }
        catch (NoSuchMethodException nsme) {
        }
        catch (Exception e) {
            Logger.logException(e);
        }
        methodLookup.put(methodLookupName, m);
        return result;
    }

    static final Class[] EMPTY_CLASS_ARRAY = new Class[0];
    static Map methodLookup = Collections.synchronizedMap(new HashMap());
}
