/*
Bridge Code to use Velocity template engine with Niggle
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote
products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.templates.velocityimpl;

import com.revusky.niggle.servlet.NiggleConfig;
import com.revusky.niggle.templates.*;
import com.revusky.oreo.util.*;
import org.apache.velocity.*;
import org.apache.velocity.context.Context;
import org.apache.velocity.io.VelocityWriter;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;


/**
 * An implementation of the com.revusky.niggle.templates.Page interface
 * that uses the Velocity template engine.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */
public class VelocityPage implements Page {

    private Template template;
    private Context context;
    private String contentType = "text/html";
    private ClassLoader classLoader; // We need this to find ResourceBundles
    private Locale locale = Locale.getDefault();

    /**
     *  Creates a new VelocityPage.
     *  @param template the template to use.
     *  @param context the WebMacro context
     *  @param cl The classloader for looking up resources.
     */
    public VelocityPage(Template template, Context context, Locale locale, ClassLoader cl)
    throws IOException {
        this.template = template;
        this.context = context;
        this.locale = locale;
        classLoader = cl;
    }

    public void outputPage(HttpServletResponse response, boolean isStatic) throws IOException {
        response.setContentType(contentType);
        response.setLocale(getLocale());
        if (!isStatic) {
            response.setHeader("Pragma", "no-cache"); // http 1.0
            response.setHeader("Cache-Control", "no-cache"); // http 1.1
        }
        try {
            template.merge(context, response.getWriter());
        }
        catch (IOException ioe) {
            throw ioe;
        }
        catch (Exception e) {
            Assert.rethrow(e);
        }
    }

    /**
     * @return the processed HTML as a String
     */
    public String toString() {
        StringWriter sw = new StringWriter();
        try {
            template.merge(context, sw);
        }
        catch (Exception e) {
            Assert.rethrow(e);
        }
        return sw.toString();
    }

    public void expose(String key, Object item) {
        context.put(key, item);
    }

    public void expose(String key, boolean b) {
        Boolean B = b ? Boolean.TRUE : Boolean.FALSE;
        context.put(key, B);
    }

    public void exposeResourceBundle(String lookupName) {
        ResourceBundle rb = null;
        Locale l = getLocale();
        if (classLoader != null) {
            rb = ResourceBundle.getBundle(lookupName, l, classLoader);
        }
        else {
            rb = ResourceBundle.getBundle(lookupName, l);
        }
        String key = lookupName.replace('.', '_');
        context.put(key, new ResourceBundleWrapper(rb));
    }

    public Locale getLocale() {
        return locale;
    }
}
