/*
Bridge Code to use Velocity template engine with Niggle
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.niggle.templates.velocityimpl;

import java.io.*;
import java.util.*;
import java.net.URL;
import javax.servlet.*;
import com.revusky.niggle.servlet.*;
import com.revusky.niggle.templates.*;
import com.revusky.oreo.util.*;
import org.apache.velocity.*;
import org.apache.velocity.app.*;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ResourceNotFoundException;


/**
 * A concrete PageFactory type that vends Page objects
 * that wrap Velocity template objects
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */

public class VelocityPageFactory implements PageFactory {
    protected ClassLoader classLoader;
    String templatePath;

    public Page getPage(String name, ServletInteraction interaction) throws IOException {
        VelocityContext vct = new VelocityContext();
        Template template = null;
        Locale locale = interaction.getLocale();
        String[] filenames = PresentationWrapper.getFilenames(name, locale);
        for (int i = 0; i<filenames.length; i++) {
            if (i==3) {
                locale = Locale.getDefault();
            }
	    String encoding = PresentationWrapper.getCharset(locale);
            template = getTemplate(filenames[i], encoding);
            if (template != null) {
                break;
            }
        }
        return new VelocityPage(template, 
                                getContext(interaction),
                                locale,
                                classLoader);
    }

    protected Template getTemplate(String name, String encoding) {
        try {
            return Velocity.getTemplate(name, encoding);
        } 
        catch (ResourceNotFoundException rnfe) {
            return null;
        }
        catch (Exception e) {
            Assert.rethrow(e);
        }
        return null;
    }

    protected Context getContext(ServletInteraction interaction) {
        VelocityContext result = new VelocityContext();
        result.put("req", new HttpServletRequestWrap(interaction.request ));
        result.put("res", new HttpServletResponseWrap(interaction.response));
        return result;
    }


    public void setConfig(NiggleConfig config) throws ServletException {
        this.classLoader = config.getServletClassLoader();
        CustomResourceLoader.setClassLoader(classLoader);
        String resourceBase = config.getInitParameter("RESOURCE_BASE");
        if (Util.hasContent(resourceBase)) {
            CustomResourceLoader.setSearchPath(resourceBase);
        }
        try {
            Velocity.setProperty("resource.loader", "niggle_custom");
            Velocity.setProperty("niggle_custom.resource.loader.class", 
                                 "com.revusky.niggle.templates.velocityimpl.CustomResourceLoader");
            Velocity.init();
        }
        catch (Exception e) {
            throw new ServletException(e);
        }
    }
}
