/*
Bridge Code to use WebMacro template engine with Niggle
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.niggle.templates.webmacroimpl;

import java.util.*;
import java.io.*;
import java.net.URL;
import org.webmacro.util.*;
import org.webmacro.*;
import org.webmacro.engine.*;
import  com.revusky.niggle.templates.PresentationWrapper;
import com.revusky.oreo.util.*;

public class WebMacroTemplate extends WMTemplate {
    private File file;
    private URL url;
    private String name, encoding;
    private long lastParsed;

    public WebMacroTemplate(String name, Broker broker, URL url, String encoding)  {
        super(broker);
        this.name = name;
        this.encoding = encoding;
        this.url = url;
        String f = url.getFile();
        if (Util.hasContent(f)) {
            if (f.startsWith("file:")) {
                f = f.substring(5);
            }
            File file = new File(f);
            if (file.exists())
                this.file = file;
        }
    }

    protected Reader getReader() throws IOException {
        InputStream stream = url.openConnection().getInputStream();
        if (stream == null)
            stream = new FileInputStream(file);
        return new BufferedReader(new InputStreamReader(stream, encoding));
    }

    public void parse() throws IOException {
        try {
            super.parse();
            lastParsed = System.currentTimeMillis();
        }
        catch (TemplateException te) {
            Assert.rethrow(te);
        }
    }

    boolean reparse(long interval) {
        if (file != null && file.exists()) {
            if (file.lastModified() > lastParsed)
                try {
                    parse();
                    return true;
                }
                catch (Exception e) {
                }
        }
        else {
            long now = System.currentTimeMillis();
            if (now - lastParsed > interval) {
                try {
                    parse();
                    return true;
                }
                catch (Exception e) {
                }
            }
        }
        return false;
    }

    public String toString() {
        return "WebMacro template: " + name;
    }
}
