/*
OREO Object-relational layer
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.oreo.metadata;

import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import com.revusky.oreo.util.*;
import com.revusky.oreo.*;
import java.io.*;
import java.util.*;

abstract public class MetadataLoader {

    static public void loadRecordMetadata(DataRegistry registry, String filename)
    throws SAXException, IOException {
        loadRecordMetadata(registry, filename, null);
    }

    static public void loadDataSources(DataRegistry registry, String filename)
    throws SAXException, IOException {
        loadDataSources(registry, filename, null);
    }

    static public void loadRecordMetadata(DataRegistry registry, String filename, ClassLoader cl)
    throws SAXException, IOException {
        Document doc = getDOM(filename, cl);
        Map recordData = new HashMap();
        Iterator recordElements = DOMUtil.getChildElementsByName(doc.getDocumentElement(), "RECORD");
        while (recordElements.hasNext()) {
            Element element = (Element) recordElements.next();
            String type = element.getAttribute("TYPE");
            if (recordData.containsKey(type))
                throw new SAXException("The type must be unique. "
                                       + type + " has already occurred.");
            RecordMetadata data = new RecordMetadata();
            data.init(element, Collections.unmodifiableMap(recordData), cl);
            recordData.put(type, data);
        }
        for (Iterator it = recordData.values().iterator(); it.hasNext();) {
            registry.registerRecordType((RecordMetadata) it.next());
        }
    }

    static public void loadDataSources(DataRegistry registry, String filename, ClassLoader cl)
    throws SAXException, IOException {
        Document doc = getDOM(filename, cl);
        Iterator dataSources = DOMUtil.getChildElementsByName(doc.getDocumentElement(), "DATASOURCE");
        while (dataSources.hasNext()) {
            Element element = (Element) dataSources.next();
            DataSourceMetadata dsm = new DataSourceMetadata();
            dsm.init(element, cl);
            registry.registerDataSource(dsm);
            Iterator aliases = DOMUtil.getChildElementsByName(element, "ALIAS");
            while (aliases.hasNext()) {
                Element e = (Element) aliases.next();
                String alias = e.getAttribute("NAME");
                registry.registerDataSourceAlias(alias, dsm);
            }
        }
    }

    static public Document getDOM(String filename, ClassLoader cl)
    throws IOException, SAXException {
        InputStream is = Util.getInputStream(filename, cl);
        if (is == null)
            throw new FileNotFoundException("could not find " + filename);
        org.apache.xerces.parsers.DOMParser parser = new org.apache.xerces.parsers.DOMParser();
        parser.setFeature("http://apache.org/xml/features/validation/dynamic", true);
//        parser.setFeature("http://xml.org/sax/features/validation", true);
        parser.setEntityResolver(new ClasspathEntityResolver(cl));
        parser.setErrorHandler(new DefaultErrorHandler());
        InputSource source = new InputSource(is);
        source.setSystemId("classpath:" + filename);
        parser.parse(source);
        return parser.getDocument();
    }

    /**
     * A main method to validate an XML configuration file against
     * its DTD.
     */
    static public void main(String[] args) {
        if (args.length <1) {
            System.err.println("Usage : java com.revusky.oreo.metadata.MetadataLoader filename");
            System.exit(-1);
        }
        try {
            Document doc = getDOM(args[0], null);
            System.exit(0);
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }
}
