/*
OREO Object-relational layer
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL) 
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors 
may be used to endorse or promote products derived from this software 
without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.oreo.metadata;

import java.io.IOException;
import com.revusky.oreo.*;
import com.revusky.oreo.util.*;
import gnu.regexp.*;

/**
 * Field descriptor that describes a field
 * that must be a valid URL
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */

public class URLField extends StringField {

    public URLField() {
        this.normalization = TRIM;
    }

    public void checkValidValue(Record rec, Object value) throws IOException {
        super.checkValidValue(rec, value);
        if ((value != null) && !isURL((String) value)) {
            throw new MangledDataException(rec, "\"" + value + "\" is not a well-formed URL.");
        }
    }

    public String normalize(String s) {
        s = super.normalize(s);
        if (!Util.hasContent(s))
            return s;
        String lc = s.toLowerCase();
        // If no protocol is specified, assume http
        if (!(lc.startsWith("ftp://") || lc.startsWith("http://") || lc.startsWith("file://"))) {
            s = "http://" + s;
        }
        return s;
    }

    // REVISIT: This is currently a bit lame. Plenty of bad URL's could get through.
    // All this does is check for illegal characters. Better than nothing.

    private boolean isURL(String s) {
        for (int i=0; i< s.length(); i++) {
            char c = s.charAt(i);
            if ((c <= ' ') || (c >= 0x7F))
                return false;
        }
        return true;
    }
}

