/*
OREO Object-relational layer
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.oreo.predicates;

import com.revusky.oreo.*;
import com.revusky.oreo.util.Assert;

/**
 * A RecordFilter type that accepts any record as long as it is of 
 * a given class.
 */

public class ClassConstraint implements RecordFilter {

    public final Class clase;

    public ClassConstraint(Class clase) {
        this.clase = clase;
        if (!clase.isInterface())
            Assert.assert(Record.class.isAssignableFrom(clase),
                          "You should specify a Record type");
    }

    public boolean accept(Record rec) {
        if (rec instanceof RecordReference) {
            rec = ((RecordReference) rec).getUnderlyingRecord();
        }
        return (clase.isInstance(rec));
    }

    public String sqlEquivalent() {
        return null;
    }

    public String recType() {
        return null;
    }
}
