/*
OREO Object-relational layer
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.oreo.predicates;

import java.util.StringTokenizer;
import java.util.Date;
import java.text.SimpleDateFormat;

/**
 * a holder for various utility routines for parsing and building SQL queries.
 */

abstract public class SQLQueryUtil {

    /**
     * @return the token right after the first occurence of "from"
     * in the query.
     */
    static public String from(String query) {
        String result = null;
        String lastTok = "";
        boolean inQuote = false;
        StringTokenizer st = new StringTokenizer(query, " \t\r\n'\\", true);
        while (st.hasMoreTokens()) {
            String tok = st.nextToken();
            if (tok.equalsIgnoreCase("from") && !inQuote) {
                while (st.hasMoreTokens()) {
                    String tok2 = st.nextToken();
                    if (tok2.trim().length() >0)
                        return tok2;
                }
                break;
            }
            lastTok = tok;
            if (tok.equals("'") && !lastTok.equals("\\"))
                inQuote = !inQuote;
        }
        return null;
    }

    /**
     * @return the string that comes after the first occurence of "where"
     */
    static public String where(String query) {
        if (query == null)
            return null;
        int index = 0;
        boolean inQuote = false;
        String lastTok = "";
        StringTokenizer st = new StringTokenizer(query, " \t\r\n'\\", true);
        while (st.hasMoreTokens()) {
            String tok = st.nextToken();
            index += tok.length();
            if (tok.equalsIgnoreCase("where") && !inQuote)
                break;
            lastTok = tok;
            if (tok.equals("'") && !lastTok.equals("\\"))
                inQuote = !inQuote;
        }
        return query.substring(index);
    }

    static public String escapeString(Object o) {
        if (o == null)
            return "NULL";
        if (o instanceof Date) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            o = sdf.format(o);
        }
        if (o instanceof String) {
            String s = (String) o;
            StringBuffer buf = new StringBuffer("'");
            for (int i = 0; i<s.length(); i++) {
                char c = s.charAt(i);
                if (c == '\'')
                    buf.append('\\');
                buf.append(c);
            }
            buf.append((char) '\'');
            return buf.toString();
        }
        else
            return o.toString();
    }

    /**
     * This is equivalent to calling matchLikeString(s, like, '@')
     */
    static public boolean matchLikeString(String s, String like) {
        return matchLikeString(s, like, '@');
    }

    /**
     * A routine that does the Ansi SQL 'LIKE' comparison.
     * @param s the string to compare
     * @param like the expression to match against
     * @escape the escape character to put a literal '%' or '_' in the expression
     * @return true if this is a match.
     */
    static public boolean matchLikeString(String s, String like, char escape) {
        if (like.length() == 0)
            return s.length() == 0;
        char firstChar = like.charAt(0);
        if (firstChar == escape) {
            return (like.charAt(1) == s.charAt(0) 
                    && matchLikeString(s.substring(1), like.substring(2)));
        }
        else if (firstChar == '%') {
            while (like.length() > 0 && like.charAt(0) == '%') {
                like = like.substring(1);
            }
            if (like.length() == 0)
                return true;
            firstChar = like.charAt(0);
            for (int i = 0; i<s.length(); i++) {
                if (firstChar != s.charAt(i) && firstChar != '_' && firstChar != escape)
                    continue;
                if (matchLikeString(s.substring(i), like, escape))
                    return true;
            }
            return false;
        }
        else  {
            for (int i = 0; i<like.length(); i++) {
                char c = like.charAt(i);
                if (c == escape || c == '%')
                    return matchLikeString(s.substring(i), like.substring(i), escape);
                if (i >= s.length())
                    break;
                if (c != '_' && c != s.charAt(i)) 
                    return false;
            }
            return s.length() == like.length();
        }
    }
}
