/*
OREO Object-relational layer
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.oreo.predicates;

import com.revusky.oreo.*;
import com.revusky.oreo.util.Assert;

/**
 * A simple RecordFilter implementation that simply 
 * makes sure that a record is of a given type.
 */
public class TypeFilter implements RecordFilter {

    private String recType, query;

    public TypeFilter(String recType) {
        this.recType = recType;
        if (recType != null) {
            try {
                RecordDescriptor rd = DataRegistryImpl.getDefaultRegistry().getMetadata(recType);
                query = "SELECT * FROM " + rd.getTableName();
            }
            catch (Exception e) {
                Assert.rethrow(e);
            }
        }
    }

    public boolean accept(Record rec) {
        if (rec == null)
            return false;
        if (recType == null) {
            return true;
        }
        return rec.getType().equals(recType);
    }

    public String recType() {
        return recType;
    }

    public String sqlEquivalent() {
        return query;
    }
}
