/*
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.oreo.util;

import java.util.StringTokenizer;

public class EmailAddress {

    private String address;

    /**
     * construct a new email address, must be 
     * a valid (looking) internet email address.
     */
    public EmailAddress(String address) {
        if (!isValidEmailAddress(address)) {
            throw new IllegalArgumentException();
        }
        this.address = address;
    }

    public String toString() {
        return address;
    }

    public int hashCode() {
        return address.hashCode();
    }

    public boolean equals(Object o) {
        if (o instanceof EmailAddress) {
            return o.toString().equalsIgnoreCase(address);
        }
        return false;
    }

    static public void main(String[] args)  {
        System.out.println(isValidEmailAddress(args[0]));
    }

    static public boolean isValidEmailAddress(String address) {
        if (address == null || address.length() < 6)
            return false;
        int i = address.indexOf("@");
        if (i < 1)
            return false;
        String user = address.substring(0, i);
        String domain = address.substring(i + 1);
        return checkUser(user) && checkDomain(domain);
    }

    static private boolean checkUser(String user) {
        int len = user.length();
        if (len <1 || len > 64)
            return false;
        char firstChar = user.charAt(0);
        char lastChar = user.charAt(len -1);
        if (firstChar == '.' || lastChar == '.')
            return false;
        return !hasForbiddenCharForUser(user); 
    }

    static private boolean checkDomain(String domain) {
        int len = domain.length();
        if (len > 256 || len < 4)
            return false;
        char firstChar = domain.charAt(0);
        char lastChar = domain.charAt(len -1);
        if (firstChar == '.' || lastChar == '.')
            return false;
        if ((domain.indexOf('.') == -1) || (domain.indexOf("..") != -1))
            return false;
        if (hasNonAlphaNumDashDot(domain))
            return false;
        StringTokenizer st = new StringTokenizer(domain, ".");
        while (st.hasMoreTokens()) {
            String tok = st.nextToken();
            int length = tok.length();
            if (!st.hasMoreTokens() && length < 2)
                return false;
            if (tok.charAt(0) == '-' || tok.charAt(length-1) == '-')
                return false;
        }
        return true;
    }

    static private boolean hasForbiddenCharForUser(final String s) {
        int len = s.length();
        while (len > 0) {
            if (permissibleChars.indexOf(s.charAt(--len)) == -1)
                return true;
        }
        return false;
    }

    static private boolean hasNonAlphaNumDashDot(final String s) {
        int len = s.length();
        while (len > 0) {
            char c = s.charAt(--len);
            if (alphaNumDash.indexOf(c) == -1)
                return true;
        }
        return false;
    }

    static private String alphaNumDash = 
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ" +
    "abcdefghijklmnopqrstuvwxyz" +
    "0123456789-.";

    static private String permissibleChars = 
    alphaNumDash + "._'+$/%";
}
