/*
OREO Object-relational layer
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote
products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.oreo.xml;

import java.io.*;
import java.util.zip.GZIPInputStream;
import org.xml.sax.*;

public class GZEntityResolver implements EntityResolver {

    File directory;
    String encoding;

    public GZEntityResolver(File directory, String encoding) {
        this.directory = directory;
        this.encoding = encoding;
    }

    public InputSource resolveEntity(final String publicId, final String systemId) throws SAXException {
        InputSource result = null;
        InputStream is = null;
        String sid = systemId;
        try {
            if (systemId.startsWith("file:")) {
                sid = sid.substring("file:".length());
                int i = 0;
                while (sid.charAt(i) == '/')
                    i++;
                if (i > 0)
                    sid = sid.substring(i-1);
            }
            File f = new File(sid);
            if (!f.exists() && !f.isAbsolute())
                f = new File(directory, sid);
            if (!f.exists()) {
                if (!f.getName().endsWith(".gz")) {
                    File gzf = new File(f.getAbsolutePath() + ".gz");
                    if (gzf.isFile())
                        f = gzf;
                }
            }
            if (f.exists()) {
                is = new FileInputStream(f);
                if (f.getName().endsWith(".gz")) {
                    is = new GZIPInputStream(is);
                }
            }
        }
        catch (IOException ioe) {
            return null;
        }
        if (is != null) {
            result = new InputSource(is);
            result.setPublicId(publicId);
            result.setSystemId(systemId);
            result.setEncoding(encoding);
        }
        return result;
    }
}